@@ -10,6 +10,9 @@ import (
 	"os/exec"
 	"syscall"
 
+	"github.com/prometheus/client_golang/prometheus"
+	"gitlab.com/gitlab-org/labkit/log"
+
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/filestore"
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/helper"
@@ -17,9 +20,20 @@ import (
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/zipartifacts"
 )
 
+var zipSubcommandsErrorsCounter = prometheus.NewCounterVec(
+	prometheus.CounterOpts{
+		Name: "gitlab_workhorse_zip_subcommand_errors_total",
+		Help: "Errors comming from subcommands used for processing ZIP archives",
+	}, []string{"error"})
+
+func init() {
+	prometheus.MustRegister(zipSubcommandsErrorsCounter)
+}
+
 type artifactsUploadProcessor struct {
-	opts   *filestore.SaveFileOpts
-	stored bool
+	opts *filestore.SaveFileOpts
+
+	upload.SavedFileTracker
 }
 
 func (a *artifactsUploadProcessor) generateMetadataFromZip(ctx context.Context, file *filestore.FileHandler) (*filestore.FileHandler, error) {
@@ -40,7 +54,7 @@ func (a *artifactsUploadProcessor) generateMetadataFromZip(ctx context.Context,
 	}
 
 	zipMd := exec.CommandContext(ctx, "gitlab-zip-metadata", fileName)
-	zipMd.Stderr = os.Stderr
+	zipMd.Stderr = log.ContextLogger(ctx).Writer()
 	zipMd.SysProcAttr = &syscall.SysProcAttr{Setpgid: true}
 	zipMd.Stdout = metaWriter
 
@@ -62,10 +76,21 @@ func (a *artifactsUploadProcessor) generateMetadataFromZip(ctx context.Context,
 	}()
 
 	if err := zipMd.Wait(); err != nil {
-		if st, ok := helper.ExitStatus(err); ok && st == zipartifacts.StatusNotZip {
+		st, ok := helper.ExitStatus(err)
+
+		if !ok {
+			return nil, err
+		}
+
+		zipSubcommandsErrorsCounter.WithLabelValues(zipartifacts.ErrorLabelByCode(st)).Inc()
+
+		if st == zipartifacts.CodeNotZip {
 			return nil, nil
 		}
-		return nil, err
+
+		if st == zipartifacts.CodeLimitsReached {
+			return nil, zipartifacts.ErrBadMetadata
+		}
 	}
 
 	metaWriter.Close()
@@ -79,10 +104,10 @@ func (a *artifactsUploadProcessor) ProcessFile(ctx context.Context, formName str
 	if formName != "file" {
 		return fmt.Errorf("invalid form field: %q", formName)
 	}
-	if a.stored {
+	if a.Count() > 0 {
 		return fmt.Errorf("artifacts request contains more than one file")
 	}
-	a.stored = true
+	a.Track(formName, file.LocalPath)
 
 	select {
 	case <-ctx.Done():
@@ -92,23 +117,23 @@ func (a *artifactsUploadProcessor) ProcessFile(ctx context.Context, formName str
 		// TODO: can we rely on disk for shipping metadata? Not if we split workhorse and rails in 2 different PODs
 		metadata, err := a.generateMetadataFromZip(ctx, file)
 		if err != nil {
-			return fmt.Errorf("generateMetadataFromZip: %v", err)
+			return err
 		}
 
 		if metadata != nil {
-			for k, v := range metadata.GitLabFinalizeFields("metadata") {
+			fields, err := metadata.GitLabFinalizeFields("metadata")
+			if err != nil {
+				return fmt.Errorf("finalize metadata field error: %v", err)
+			}
+
+			for k, v := range fields {
 				writer.WriteField(k, v)
 			}
+
+			a.Track("metadata", metadata.LocalPath)
 		}
 	}
-	return nil
-}
 
-func (a *artifactsUploadProcessor) ProcessField(ctx context.Context, formName string, writer *multipart.Writer) error {
-	return nil
-}
-
-func (a *artifactsUploadProcessor) Finalize(ctx context.Context) error {
 	return nil
 }
 
@@ -116,10 +141,15 @@ func (a *artifactsUploadProcessor) Name() string {
 	return "artifacts"
 }
 
-func UploadArtifacts(myAPI *api.API, h http.Handler) http.Handler {
+func UploadArtifacts(myAPI *api.API, h http.Handler, p upload.Preparer) http.Handler {
 	return myAPI.PreAuthorizeHandler(func(w http.ResponseWriter, r *http.Request, a *api.Response) {
-		mg := &artifactsUploadProcessor{opts: filestore.GetOpts(a)}
+		opts, _, err := p.Prepare(a)
+		if err != nil {
+			helper.Fail500(w, r, fmt.Errorf("UploadArtifacts: error preparing file storage options"))
+			return
+		}
 
-		upload.HandleFileUploads(w, r, h, a, mg)
+		mg := &artifactsUploadProcessor{opts: opts, SavedFileTracker: upload.SavedFileTracker{Request: r}}
+		upload.HandleFileUploads(w, r, h, a, mg, opts)
 	}, "/authorize")
 }
