@@ -9,7 +9,12 @@ import (
 	"os"
 	"strconv"
 
+	"github.com/dgrijalva/jwt-go"
+
+	"gitlab.com/gitlab-org/labkit/log"
+
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/objectstore"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/secret"
 )
 
 type SizeError error
@@ -39,6 +44,11 @@ type FileHandler struct {
 	hashes map[string]string
 }
 
+type uploadClaims struct {
+	Upload map[string]string `json:"upload"`
+	jwt.StandardClaims
+}
+
 // SHA256 hash of the handled file
 func (fh *FileHandler) SHA256() string {
 	return fh.hashes["sha256"]
@@ -50,8 +60,10 @@ func (fh *FileHandler) MD5() string {
 }
 
 // GitLabFinalizeFields returns a map with all the fields GitLab Rails needs in order to finalize the upload.
-func (fh *FileHandler) GitLabFinalizeFields(prefix string) map[string]string {
+func (fh *FileHandler) GitLabFinalizeFields(prefix string) (map[string]string, error) {
+	// TODO: remove `data` these once rails fully and exclusively support `signedData` (https://gitlab.com/gitlab-org/gitlab-workhorse/-/issues/263)
 	data := make(map[string]string)
+	signedData := make(map[string]string)
 	key := func(field string) string {
 		if prefix == "" {
 			return field
@@ -60,24 +72,30 @@ func (fh *FileHandler) GitLabFinalizeFields(prefix string) map[string]string {
 		return fmt.Sprintf("%s.%s", prefix, field)
 	}
 
-	if fh.Name != "" {
-		data[key("name")] = fh.Name
-	}
-	if fh.LocalPath != "" {
-		data[key("path")] = fh.LocalPath
-	}
-	if fh.RemoteURL != "" {
-		data[key("remote_url")] = fh.RemoteURL
-	}
-	if fh.RemoteID != "" {
-		data[key("remote_id")] = fh.RemoteID
-	}
-	data[key("size")] = strconv.FormatInt(fh.Size, 10)
-	for hashName, hash := range fh.hashes {
-		data[key(hashName)] = hash
+	for k, v := range map[string]string{
+		"name":       fh.Name,
+		"path":       fh.LocalPath,
+		"remote_url": fh.RemoteURL,
+		"remote_id":  fh.RemoteID,
+		"size":       strconv.FormatInt(fh.Size, 10),
+	} {
+		data[key(k)] = v
+		signedData[k] = v
 	}
 
-	return data
+	for hashName, hash := range fh.hashes {
+		data[key(hashName)] = hash
+		signedData[hashName] = hash
+	}
+
+	claims := uploadClaims{Upload: signedData, StandardClaims: secret.DefaultClaims}
+	jwtData, err := secret.JWTTokenString(claims)
+	if err != nil {
+		return nil, err
+	}
+	data[key("gitlab-workhorse-upload")] = jwtData
+
+	return data, nil
 }
 
 // SaveFileFromReader persists the provided reader content to all the location specified in opts. A cleanup will be performed once ctx is Done
@@ -99,7 +117,16 @@ func SaveFileFromReader(ctx context.Context, reader io.Reader, size int64, opts
 		}
 	}()
 
-	if opts.IsMultipart() {
+	useS3Client := opts.UseWorkhorseClientEnabled() && opts.ObjectStorageConfig.IsAWS() && opts.ObjectStorageConfig.IsValid()
+
+	if useS3Client {
+		remoteWriter, err = objectstore.NewS3Object(ctx, opts.RemoteTempObjectID, opts.ObjectStorageConfig.S3Credentials, opts.ObjectStorageConfig.S3Config, opts.Deadline)
+		if err != nil {
+			return nil, err
+		}
+
+		writers = append(writers, remoteWriter)
+	} else if opts.IsMultipart() {
 		remoteWriter, err = objectstore.NewMultipart(ctx, opts.PresignedParts, opts.PresignedCompleteMultipart, opts.PresignedAbortMultipart, opts.PresignedDelete, opts.PutHeaders, opts.Deadline, opts.PartSize)
 		if err != nil {
 			return nil, err
@@ -138,6 +165,24 @@ func SaveFileFromReader(ctx context.Context, reader io.Reader, size int64, opts
 		return nil, SizeError(fmt.Errorf("expected %d bytes but got only %d", size, fh.Size))
 	}
 
+	logger := log.WithContextFields(ctx, log.Fields{
+		"copied_bytes":     fh.Size,
+		"is_local":         opts.IsLocal(),
+		"is_multipart":     opts.IsMultipart(),
+		"is_remote":        opts.IsRemote(),
+		"remote_id":        opts.RemoteID,
+		"temp_file_prefix": opts.TempFilePrefix,
+		"use_s3_client":    useS3Client,
+	})
+
+	if opts.IsLocal() {
+		logger = logger.WithField("local_temp_path", opts.LocalTempPath)
+	} else if useS3Client {
+		logger = logger.WithField("remote_temp_object", opts.RemoteTempObjectID)
+	}
+
+	logger.Info("saved file")
+
 	fh.hashes = hashes.finish()
 
 	if opts.IsRemote() {
