@@ -5,6 +5,7 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
@@ -15,6 +16,9 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/dgrijalva/jwt-go"
+	"github.com/stretchr/testify/require"
+
 	"gitlab.com/gitlab-org/labkit/log"
 
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/secret"
@@ -77,6 +81,14 @@ func AssertResponseWriterHeader(t *testing.T, w http.ResponseWriter, header stri
 	assertHeaderExists(t, header, actual, expected)
 }
 
+func AssertAbsentResponseWriterHeader(t *testing.T, w http.ResponseWriter, header string) {
+	actual := w.Header()[http.CanonicalHeaderKey(header)]
+
+	if len(actual) != 0 {
+		t.Fatalf("for HTTP request expected not to receive the header %q, got %+v", header, actual)
+	}
+}
+
 func AssertResponseHeader(t *testing.T, w interface{}, header string, expected ...string) {
 	var actual []string
 
@@ -166,3 +178,32 @@ func LoadFile(t *testing.T, filePath string) string {
 	}
 	return string(content)
 }
+
+func ReadAll(t *testing.T, r io.Reader) []byte {
+	t.Helper()
+
+	b, err := ioutil.ReadAll(r)
+	require.NoError(t, err)
+	return b
+}
+
+func ParseJWT(token *jwt.Token) (interface{}, error) {
+	// Don't forget to validate the alg is what you expect:
+	if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
+		return nil, fmt.Errorf("unexpected signing method: %v", token.Header["alg"])
+	}
+
+	ConfigureSecret()
+	secretBytes, err := secret.Bytes()
+	if err != nil {
+		return nil, fmt.Errorf("read secret from file: %v", err)
+	}
+
+	return secretBytes, nil
+}
+
+// UploadClaims represents the JWT claim for upload parameters
+type UploadClaims struct {
+	Upload map[string]string `json:"upload"`
+	jwt.StandardClaims
+}
