@@ -14,7 +14,7 @@ import (
 	"strings"
 	"testing"
 
-	jwt "github.com/dgrijalva/jwt-go"
+	"github.com/dgrijalva/jwt-go"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
@@ -42,7 +42,7 @@ func testArtifactsUpload(t *testing.T, uploadArtifacts uploadArtifactsFunction)
 	reqBody, contentType, err := multipartBodyWithFile()
 	require.NoError(t, err)
 
-	ts := uploadTestServer(t, nil)
+	ts := signedUploadTestServer(t, nil)
 	defer ts.Close()
 
 	ws := startWorkhorseServer(ts.URL)
@@ -60,20 +60,30 @@ func TestArtifactsUpload(t *testing.T) {
 	testArtifactsUpload(t, uploadArtifactsV4)
 }
 
+func expectSignedRequest(t *testing.T, r *http.Request) {
+	t.Helper()
+
+	_, err := jwt.Parse(r.Header.Get(secret.RequestHeader), testhelper.ParseJWT)
+	require.NoError(t, err)
+}
+
 func uploadTestServer(t *testing.T, extraTests func(r *http.Request)) *httptest.Server {
 	return testhelper.TestServerWithHandler(regexp.MustCompile(`.`), func(w http.ResponseWriter, r *http.Request) {
 		if strings.HasSuffix(r.URL.Path, "/authorize") {
+			expectSignedRequest(t, r)
+
 			w.Header().Set("Content-Type", api.ResponseContentType)
 			if _, err := fmt.Fprintf(w, `{"TempPath":"%s"}`, scratchDir); err != nil {
 				t.Fatal(err)
 			}
 			return
 		}
+
 		err := r.ParseMultipartForm(100000)
 		if err != nil {
 			t.Fatal(err)
 		}
-		nValues := 7 // file name, path, size, md5, sha1, sha256, sha512 for just the upload (no metadata because we are not POSTing a valid zip file)
+		nValues := 10 // file name, path, remote_url, remote_id, size, md5, sha1, sha256, sha512, gitlab-workhorse-upload for just the upload (no metadata because we are not POSTing a valid zip file)
 		if len(r.MultipartForm.Value) != nValues {
 			t.Errorf("Expected to receive exactly %d values", nValues)
 		}
@@ -87,52 +97,85 @@ func uploadTestServer(t *testing.T, extraTests func(r *http.Request)) *httptest.
 	})
 }
 
-func parseJWT(token *jwt.Token) (interface{}, error) {
-	// Don't forget to validate the alg is what you expect:
-	if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
-		return nil, fmt.Errorf("Unexpected signing method: %v", token.Header["alg"])
-	}
+func signedUploadTestServer(t *testing.T, extraTests func(r *http.Request)) *httptest.Server {
+	t.Helper()
 
-	testhelper.ConfigureSecret()
-	secretBytes, err := secret.Bytes()
-	if err != nil {
-		return nil, fmt.Errorf("read secret from file: %v", err)
-	}
+	return uploadTestServer(t, func(r *http.Request) {
+		expectSignedRequest(t, r)
 
-	return secretBytes, nil
+		if extraTests != nil {
+			extraTests(r)
+		}
+	})
 }
 
 func TestAcceleratedUpload(t *testing.T) {
-	ts := uploadTestServer(t, func(r *http.Request) {
-		jwtToken, err := jwt.Parse(r.Header.Get(upload.RewrittenFieldsHeader), parseJWT)
-		require.NoError(t, err)
-
-		rewrittenFields := jwtToken.Claims.(jwt.MapClaims)["rewritten_fields"].(map[string]interface{})
-		if len(rewrittenFields) != 1 || len(rewrittenFields["file"].(string)) == 0 {
-			t.Fatalf("Unexpected rewritten_fields value: %v", rewrittenFields)
-		}
-	})
-
-	defer ts.Close()
-	ws := startWorkhorseServer(ts.URL)
-	defer ws.Close()
-
-	resources := []string{
-		`/example`,
-		`/uploads/personal_snippet`,
-		`/uploads/user`,
-		`/api/v4/projects/1/wikis/attachments`,
-		`/api/graphql`,
+	tests := []struct {
+		method             string
+		resource           string
+		signedFinalization bool
+	}{
+		{"POST", `/example`, false},
+		{"POST", `/uploads/personal_snippet`, true},
+		{"POST", `/uploads/user`, true},
+		{"POST", `/api/v4/projects/1/wikis/attachments`, false},
+		{"POST", `/api/graphql`, false},
+		{"PUT", "/api/v4/projects/9001/packages/nuget/v1/files", true},
+		{"POST", `/api/v4/groups/import`, true},
+		{"POST", `/api/v4/projects/import`, true},
+		{"POST", `/import/gitlab_project`, true},
+		{"POST", `/import/gitlab_group`, true},
+		{"POST", `/api/v4/projects/9001/packages/pypi`, true},
 	}
-	for _, resource := range resources {
-		reqBody, contentType, err := multipartBodyWithFile()
-		require.NoError(t, err)
 
-		resp, err := http.Post(ws.URL+resource, contentType, reqBody)
-		require.NoError(t, err)
-		require.Equal(t, 200, resp.StatusCode)
+	for _, tt := range tests {
+		t.Run(tt.resource, func(t *testing.T) {
+			ts := uploadTestServer(t,
+				func(r *http.Request) {
+					if tt.signedFinalization {
+						expectSignedRequest(t, r)
+					}
 
-		resp.Body.Close()
+					token, err := jwt.ParseWithClaims(r.Header.Get(upload.RewrittenFieldsHeader), &upload.MultipartClaims{}, testhelper.ParseJWT)
+					require.NoError(t, err)
+
+					rewrittenFields := token.Claims.(*upload.MultipartClaims).RewrittenFields
+					if len(rewrittenFields) != 1 || len(rewrittenFields["file"]) == 0 {
+						t.Fatalf("Unexpected rewritten_fields value: %v", rewrittenFields)
+					}
+
+					token, jwtErr := jwt.ParseWithClaims(r.PostFormValue("file.gitlab-workhorse-upload"), &testhelper.UploadClaims{}, testhelper.ParseJWT)
+					require.NoError(t, jwtErr)
+
+					uploadFields := token.Claims.(*testhelper.UploadClaims).Upload
+					require.Contains(t, uploadFields, "name")
+					require.Contains(t, uploadFields, "path")
+					require.Contains(t, uploadFields, "remote_url")
+					require.Contains(t, uploadFields, "remote_id")
+					require.Contains(t, uploadFields, "size")
+					require.Contains(t, uploadFields, "md5")
+					require.Contains(t, uploadFields, "sha1")
+					require.Contains(t, uploadFields, "sha256")
+					require.Contains(t, uploadFields, "sha512")
+				})
+
+			defer ts.Close()
+			ws := startWorkhorseServer(ts.URL)
+			defer ws.Close()
+
+			reqBody, contentType, err := multipartBodyWithFile()
+			require.NoError(t, err)
+
+			req, err := http.NewRequest(tt.method, ws.URL+tt.resource, reqBody)
+			require.NoError(t, err)
+
+			req.Header.Set("Content-Type", contentType)
+			resp, err := http.DefaultClient.Do(req)
+			require.NoError(t, err)
+			require.Equal(t, 200, resp.StatusCode)
+
+			resp.Body.Close()
+		})
 	}
 }
 
@@ -214,19 +257,15 @@ func TestLfsUpload(t *testing.T) {
 		require.Equal(t, r.Method, "PUT")
 		switch r.RequestURI {
 		case resource + "/authorize":
-			// Expect the authorization call to be signed
-			_, err := jwt.Parse(r.Header.Get(secret.RequestHeader), parseJWT)
-			require.NoError(t, err)
+			expectSignedRequest(t, r)
 
 			// Instruct workhorse to accept the upload
 			w.Header().Set("Content-Type", api.ResponseContentType)
-			_, err = fmt.Fprint(w, lfsApiResponse)
+			_, err := fmt.Fprint(w, lfsApiResponse)
 			require.NoError(t, err)
 
 		case resource:
-			// Expect the finalization call to be signed
-			_, err := jwt.Parse(r.Header.Get(secret.RequestHeader), parseJWT)
-			require.NoError(t, err)
+			expectSignedRequest(t, r)
 
 			// Expect the request to point to a file on disk containing the data
 			require.NoError(t, r.ParseForm())
@@ -273,16 +312,16 @@ func packageUploadTestServer(t *testing.T, resource string, reqBody string, rspB
 		)
 		switch r.RequestURI {
 		case resource + "/authorize":
-			// Expect the authorization call to be signed
-			_, err := jwt.Parse(r.Header.Get(secret.RequestHeader), parseJWT)
-			require.NoError(t, err)
+			expectSignedRequest(t, r)
 
 			// Instruct workhorse to accept the upload
 			w.Header().Set("Content-Type", api.ResponseContentType)
-			_, err = fmt.Fprint(w, apiResponse)
+			_, err := fmt.Fprint(w, apiResponse)
 			require.NoError(t, err)
 
 		case resource:
+			expectSignedRequest(t, r)
+
 			// Expect the request to point to a file on disk containing the data
 			require.NoError(t, r.ParseForm())
 
@@ -330,7 +369,6 @@ func testPackageFileUpload(t *testing.T, resource string) {
 func TestPackageFilesUpload(t *testing.T) {
 	routes := []string{
 		"/api/v4/packages/conan/v1/files",
-		"/api/v4/projects/9001/packages/nuget/v1/files",
 		"/api/v4/projects/2412/packages/maven/v1/files",
 	}
 
